/****************************************************************
Project : PPE Robot Solaire
Version : 1.2.0
// --------------------------------------------------------------
Fichier source : mainTestSISIV2.c
Date de  cration : 06/01/2010
Mise  jour le : 22/05/2011
// --------------------------------------------------------------
Author  : Philippe Mariano
Company : Lyce Pierre Emile Martin 18000 BOURGES
// --------------------------------------------------------------
Comments:
- Mise en oeuvre du protocole I2C sur l'interface TWI
(Adaptation de la note d'application AVR315 d'ATMEL)
- Commande d'un afficheur LCD Batron I2C (PCF2119x)
- Lecture des BP et recopie sur les LED.
- Test de la carte MD23/MD25(Reglage et commande des moteurs en
Mode0, lecture et affichage des registres, test de dplacements)
- Test de la carte SD21
- Test de la boussole HMC6352
- Dplacement et orientation au sud
// --------------------------------------------------------------
Cross compilateur: CVAVR 2.04.08a
// --------------------------------------------------------------
Chip type           : ATmega32
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 512
****************************************************************/
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// Informations sur le paramtrage du programme
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// Le nombre de dplacement est dfini par 
// #define Nombre_Deplacements_du_Balayage             ?
// Les paramtres des dplacements doivent tre mmoriss dans le tableau  
// Table_Balayage[Nombre_Deplacements_du_Balayage][4]
// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// ===========================================================================================================================
// Partie dclarative du programme
// ===========================================================================================================================
// Bibliothques et dfinitions
// ===========================================================================================================================
#include <mega32.h>
// ===========================================================================================================================
// Bibliothques CVAVR ( rajouter manuellement)
// ===========================================================================================================================
#include <stdlib.h>
#include <stdio.h>				// Fonction sprintf
#include <math.h>
#include <string.h>				// Fonctions de traitement des chanes de caractres
#include <delay.h>				// Fonction temporisation
// ===========================================================================================================================
// Bibliothques dveloppes pour la carte SISI ( rajouter manuellement)
// ===========================================================================================================================
#include "DriverTWICV2.h"       // Fonctions du drivers TWI adaptes pour CVAVR
                                // ATTENTION: Vrifier que la taille du buffer est adapte au message (20 par dfaut)
#include "stdmno.h"             // Fonctions LSBYTE et MSBYTE
#include "MD23.h"				// Dfinitions des registres de la carte moteur MD23
#include "LCDBATRON.h"          // Fonctions de lecture/criture des priphriques I2C
#include "SISISOL.h"            // Gestion de l'IHM SISI
#include "SD21.h"				// Dfinitions des registres de la carte servo SD21
#include "TableRob.h"           // Dclaration des tables de valeurs envoyes aux servos
#include "HMC6352.h"			// Dfinition des registres de la boussole Honeywell HMC6352
// ===========================================================================================================================
// Dclarations
// ===========================================================================================================================
// Redfinition des adresses (lecture / criture) de la boussole HMC6352 (0x42 par dfaut: en conflit avec les BP de la carte
// SISI si l'IHM ne dispose pas du rglage des adresses des PCF8574 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! )
// ---------------------------------------------------------------------------------------------------------------------------
#define NEW_HMC6352_SLA_W 	0x44 // Ox42 est utilis par le PCF8574 reli aux LEDs sur l'IHM SISIV1 ! 
#define	NEW_HMC6352_SLA_R   0x45 
// ---------------------------------------------------------------------------------------------------------------------------
#ifndef
#define Null	            0
#endif	
// ---------------------------------------------------------------------------------------------------------------------------
// Alias pour la trame de commande des Servos connects  la carte SD21
// ---------------------------------------------------------------------------------------------------------------------------
#define NumServo			  Frame_I2C_Servo[1]
#define RegSpeedValue	      Frame_I2C_Servo[2] 
#define RegPosLByte			  Frame_I2C_Servo[3] 
#define RegPosHByte			  Frame_I2C_Servo[4]
// ---------------------------------------------------------------------------------------------------------------------------
// Alias pour la trame de commande des moteurs connects  de la carte MD23 - 25
// ---------------------------------------------------------------------------------------------------------------------------
#define	MoteurG			        0
#define	MoteurD			        1
// ---------------------------------------------------------------------------------------------------------------------------
// Nombre de dplacements pendant le balayage

#define Nombre_Deplacements_du_Balayage             16             
#define CodeurG                                     0
#define CodeurD                                     1
// ===========================================================================================================================

// ===========================================================================================================================
// ===========================================================================================================================
// Constantes en flash
// ---------------------------------------------------------------------------------------------------------------------------
// Structure pour l'affichage du nom des registres  (A terminer)
// flash char Table_Name_Register_MD23[3][7]={"Speed1", "Speed2","Enc1a "};   // changer par une structure
// ---------------------------------------------------------------------------------------------------------------------------
// Messages textuels prdfinis
// ATTENTION: imprativement 16 caractres
// ---------------------------------------------------------------------------------------------------------------------------
flash char *Texte_Accueil_Ligne1 	= "   Test SISI    ";
flash char *Texte_Accueil_Ligne2 	= "[RBSOL 25/05/11]";
flash char *TexteMenuLedBp			= "[1]Test BP->Led ";
flash char *TexteMenuMD23			= "[2]Test MD23    ";
flash char *TexteMenuSD21			= "[3]Test SD21    ";
flash char *TexteMenuHMC6352        = "[4]HMC6352      ";
flash char *Ligne0MenuMode			= "[20]Choix Modes ";
flash char *Ligne0MenuInfos			= "[21]Aff Infos   ";
flash char *Ligne0MenuDeplacements  = "[22]Choix Depl. ";
flash char *TexteTestSD21           = "Cde Servo 1->4  ";
flash char *TexteTest1BpsurLed	    = "Recopie Bp->Led ";
flash char *Ligne1PlusMoins			= "[+/-]   [Echap] ";
flash char *Ligne1ChoixEntr			= "[Entr]          ";
flash char *Ligne1ChoixEchap		= "        [Echap] ";
flash char *Ligne1ChoixEchap5s		= "    [Echap 5s!] ";
flash char *Ligne1ChoixEchapOk		= "[Entr]  [Echap] ";
flash char *Ligne1ChoixEchapSET		= "[SET]   [Echap] ";
flash char *BpPlusSurD0				= "+ -> D0 [Echap] ";
flash char *BpFlecheHSurD1			= "FH-> D1 [Echap] ";
flash char *BpFlecheHSurD2			= "FB-> D2 [Echap] ";
flash char *BpMoinsSurD3			= "- -> D3 [Echap] ";
flash char *BpSetSurD4				= "SET->D4 [Echap] ";
flash char *BpOkSurD5				= "Ok-> D5 [Echap] ";
flash char *BpEntrSurD6				= "ENT->D6 [Echap] ";
flash char *BpEchSurD7				= "ECH->D7 [Echap] ";
// ===========================================================================================================================

// ===========================================================================================================================
// ===========================================================================================================================
// Prototypes des fonctions en cours de test
// ---------------------------------------------------------------------------------------------------------------------------
// ===========================================================================================================================

// ===========================================================================================================================
// ===========================================================================================================================
// ------------------------------------------ Declare your global variables here ---------------------------------------------
// ---------------------------------------------------------------------------------------------------------------------------
// Type				Nom														Description
// ---------------------------------------------------------------------------------------------------------------------------
// Etats du graphe des transitions "Menus"
// ---------------------------------------------------------------------------------------------------------------------------
// Enumration
enum TMenu {Menu1_LED_BP, Menu2_MD23, Menu3_SD21, Menu4_HMC6352, Menu20_Mode_MD23, Menu21_Infos_MD23, Menu22_Deplacements, 
Copie_BP_Sur_Led, Choix_Mode_MD23, Reglage_Valeurs_Mode0_MD23, Info_All, Info_Version,  Info_TensionBatt, 
Selection_Deplacements, Test_Deplacements_Elementaires, Balayage, Orientation_Sud, Test_HMC6352, Test_SD21} 
Menu=Menu1_LED_BP, Menu_1=Menu2_MD23;
// ---------------------------------------------------------------------------------------------------------------------------
// Boolen
bit Choix_Moteur=0;			// Permet de permuter le rglage de la vitesse des moteurs dans le menu mode 0
// ===========================================================================================================================


// ===========================================================================================================================
// ===========================================================================================================================
// Partie excutive du programme
// ===========================================================================================================================
// ===========================================================================================================================
void main(void)
{
// ===========================================================================================================================
// ---------------------------------------------- Declare your local variables here ------------------------------------------
// ===========================================================================================================================
// Messages pour les composants I2C
// ===========================================================================================================================
// 1) Messages prdfinis pour les ports I2C PCF8574 associs aux BP et aux LEDs de la carte S_ISI
// ---------------------------------------------------------------------------------------------------------------------------
// Type 	   		Nom 				       						Description
// ---------------------------------------------------------------------------------------------------------------------------
char Frame_I2C_LED_All_Off[]={Add_LED_I2C_W,0xff}; 		// Trame d'extinction des LED {SLA+W=0x40, Data = all Led off}
char Frame_I2C_LED_Switching[]={Add_LED_I2C_W,0x00}; 	// Trame de commande des LED {SLA+W=0x40, Data : Led_On=0
                                                        // or Off=1}. Initialisation = All On.
char Frame_I2C_BP_Read[]={Add_BP_I2C_R,0x00}; 			// Trame de lecture des BP {SLA+R=0x43, Data=Nul}
// ===========================================================================================================================
// 2) Liste de messages prdfinis pour le LCD I2C BATRON de la carte SISI
// ---------------------------------------------------------------------------------------------------------------------------
// Type 	   		Nom 				       						Description
// ---------------------------------------------------------------------------------------------------------------------------
//unsigned char Frame_I2C_LCD_BATRON_Clear[]={0x76,0x00,0x01}; 	// Effacement de l'cran {SLA+W=0x76, acces=RegIR, cde=clear}
char Frame_I2C_Lcd_Line[19]={0x76,0x40};      	    	        // Trame de texte compose de: {SLA+W=0x76, cde=write to 
						 	     								// DDRAM, le texte  afficher = 16 caractres max, Nul }
                                                            	// Cette trame est remplie avec le tableau String_to_LCD
// ===========================================================================================================================
// 3) Liste de messages prdfinis pour la carte MD23
// ===========================================================================================================================
// 30) Initialisation
unsigned char Reset_Encodeurs_MD23[]={MD23_ADD_BASE_W,Reg_Commande, Reset_Encodeur}; // Remise  zro des encodeurs
// ---------------------------------------------------------------------------------------------------------------------------
// 31) Modes
unsigned char Mode0_MD23[]={MD23_ADD_BASE_W,Reg_Mode,Mode0};    // Rglage du mode 0
// ---------------------------------------------------------------------------------------------------------------------------
// 32) Vitesse des moteurs
unsigned char Speed1[]={MD23_ADD_BASE_W,Reg_Speed1, 128};		// Arrt moteur gauche dans le mode 0
unsigned char Speed2[]={MD23_ADD_BASE_W,Reg_Speed2_Turn, 128};  // Arrt moteur droit dans le mode 0
unsigned char Time_OUT_Desable[]={MD23_ADD_BASE_W,Reg_Commande, Desactive_Timeout_Moteurs}; // Dsactivation de la scurit
                                                                // Arrt moteurs si aucune trame n'est reu pendant 2s
// ---------------------------------------------------------------------------------------------------------------------------
// 33) Lecture des registres
unsigned char Read_Reg_MD23_W[]= {MD23_ADD_BASE_W,0};		    // Premire trame  envoyer:
															    // { SLA_W=0xB0, N registre o commence la lecture }
// ===========================================================================================================================
// 4) Message pour la commande des servos
// ---------------------------------------------------------------------------------------------------------------------------
// Cette trame est compose de: {@SD21, NumServo, RegSpeedValue, RegPosLByte, RegPosHByte}
unsigned char Frame_I2C_Servo[]={SD21_ADD_BASE_W,Servo1,0,0xE8,0x03};		 

// Dtail du contenu de la "trame I2C Servo" 
// @SD21			  SLA+W = 0xC2: Adresse de la carte SD21 en criture
// NumServo			  N du registre vitesse du servo  commander, dfinit dans SD21.h (Servo1 par dfaut)
// RegSpeedValue	  Valeur  placer dans le le registre vitesse (Vitesse maxi par dfaut)
// RegPosLByte		  Octet de poids faible du mot de commande de la position (Position mini par dfaut)
// RegPosHByte		  Octet de poids fort du mot de commande de la position 
// ===========================================================================================================================
// 5) Messages pour la boussole HMC6352
// ---------------------------------------------------------------------------------------------------------------------------
// Lecture de la direction (comprise entre 0 et 3599 dixime de degrs, code sur deux octets)
unsigned char Frame_I2C_Write_Command_HMC6352[] = {NEW_HMC6352_SLA_W,'A'};      // Commande  
unsigned char Frame_I2C_Read_Data_HMC6352[] = {NEW_HMC6352_SLA_R,Null,Null};    // La boussole renvoie deux octets (MSByte et 
                                                                                // LSByte)
// ===========================================================================================================================
// Variables diverses
// ===========================================================================================================================
// Type 			Nom 				       			   				Description
// ===========================================================================================================================
// 1) Gestion de la carte MD23
// ---------------------------------------------------------------------------------------------------------------------------
// Variables utilises pour la gestion de l'affichage sur le LCD:  limitent son rafrachissement!
unsigned int Num_Reg_MD23_1 = 0;
unsigned char Vitesse_MD23_1 = 1,  BP_I2C_1 = 1, Finesse_1 = 1;
unsigned char SpeedMot1_1 = 128, SpeedMot2_1 = 128;
signed char Choix_Mode_1 = 1;
unsigned char Ubatt_1=0;
unsigned char Version_1=0;
long int TableCodeur_1[2] = {0,0};
unsigned char SelecDeplacement_1 = 0;
// ---------------------------------------------------------------------------------------------------------------------------
// Variables utilises pour la gestion de la carte MD23/25
unsigned int Num_Reg_MD23 = 0;                        		        
unsigned char Vitesse_MD23 = 0, BP_I2C = 0,  Finesse = 5 ;
unsigned char SpeedMot1 = 128, SpeedMot2 = 128 ;        // Arrt des moteurs
signed char Choix_Mode = 0;
unsigned char Ubatt = 0;				                // Tension alimentation carte MD23/MD25
unsigned char Version = 0;                              // Version du soft de la carte MD23/MD25
long int TableCodeur[2] = {0,0};                        // Reprsentatif de la valeur contenue dans les encodeurs 
                                                        // TableCodeur[0] <- Codeur1(Gauche)   TableCodeur[1] <- Codeur2 (Droit)
long int Valeur_Codeur_Stop = 0;                        // Valeur teste lors d'un dplacement lmentaire 
unsigned char NumeroCodeur = 0; 
// ---------------------------------------------------------------------------------------------------------------------------
// Variable utilises pour les test de dplacement         
unsigned char SelecDeplacement = 0;                     // Fait voluer le menu dplacement
// ---------------------------------------------------------------------------------------------------------------------------
// ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Table des valeurs utilises pour les diffrents dplacements lors d'un balayage. Valeurs dtermines avec le fichier Excel
// "Calcul_parametres_MD23_25_deplacements.xls"
// Le nombre de ligne est identifi en haut du fichier source par "Nombre_Deplacements_du_Balayage" 
// --------------------------------------------------------------------------------------------------------------------------- 
/* Essai 1 ok                                      
unsigned int Table_Balayage[Nombre_Deplacements_du_Balayage][4] = 
                                     {  // Speed1,Speed2,ValeurCodeurStop, CodeurRfrent
                                        {151,151,2292,CodeurG},    // Dplacement 1 : Ligne droite, D = 2m, V = 20cm/s
                                        {173,128,1656,CodeurG},    // Dplacement 2 : Virage 180D, R = 23cm , V = 20cm/s
                                        {151,151,1719,CodeurG},    // Dplacement 3 : Ligne droite, D = 1,5m, V = 20cm/s 
                                        {128,173,1656,CodeurD},    // Dplacement 4 : Virage 180G, R = 23cm , V = 20cm/s
                                        {128,173,828,CodeurD},     // Dplacement 5 : Virage 90G, R = 23cm , V = 20cm/s
                                        {128,128,100,CodeurG}      // Arrt !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                                     }; 
*/
// Essai 2 : Cartech'ino
unsigned int Table_Balayage[Nombre_Deplacements_du_Balayage][4] = 
                                     {  // Speed1, Speed2, ValeurCodeurStop, CodeurRfrent
                                        {151,151,5137,CodeurG},    // Dplacement 1 : Ligne droite, D = 4,5m, V = 20cm/s
                                        {171,130,1768,CodeurG},    // Dplacement 2 : Virage 180D, R = 25cm , V = 20cm/s
                                        {151,151,4584,CodeurG},    // Dplacement 3 : Ligne droite, D = 4m, V = 20cm/s 
                                        {130,171,1768,CodeurD},    // Dplacement 4 : Virage 180G, R = 25cm , V = 20cm/s
                                        {151,151,4584,CodeurG},    // Dplacement 5 : Ligne droite, D = 4m, V = 20cm/s
                                        {171,130,1768,CodeurG},    // Dplacement 6 : Virage 180D, R = 25cm , V = 20cm/s
                                        {151,151,4584,CodeurG},    // Dplacement 7 : Ligne droite, D = 4m, V = 20cm/s 
                                        {130,171,1768,CodeurD},    // Dplacement 8 : Virage 180G, R = 25cm , V = 20cm/s 
                                        {151,151,4584,CodeurG},    // Dplacement 9 : Ligne droite, D = 4m, V = 20cm/s
                                        {171,130,1768,CodeurG},    // Dplacement 10 : Virage 180D, R = 25cm , V = 20cm/s
                                        {151,151,4584,CodeurG},    // Dplacement 11: Ligne droite, D = 4m, V = 20cm/s 
                                        {130,171,1768,CodeurD},    // Dplacement 12 : Virage 180G, R = 25cm , V = 20cm/s 
                                        {151,151,4584,CodeurG},    // Dplacement 13 : Ligne droite, D = 4m, V = 20cm/s
                                        {171,130,1768,CodeurG},    // Dplacement 14 : Virage 180D, R = 25cm , V = 20cm/s
                                        {151,151,5137,CodeurG},    // Dplacement 15: Ligne droite, D = 4,5m, V = 20cm/s 
                                        {128,128,100,CodeurG}      // Arrt !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                                     }; 
// Speed1 = Vitesse moteur Gauche, 
// Speed2 = Vitesse moteur Droit, 
// ValeurCodeurStop = Valeur maximum que doit atteindre le codeur rfrent avant un changement de mouvement
// CodeurRfrent = Codeur test pour arrter le mouvement  
// ---------------------------------------------------------------------------------------------------------------------------
// /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////// 
// ---------------------------------------------------------------------------------------------------------------------------
char AfficheDeplacement1[11];         // Tableau permettant d'afficher le contenu du codeur Gauche
char AfficheDeplacement2[11];         // Tableau permettant d'afficher le contenu du codeur Droit
// ---------------------------------------------------------------------------------------------------------------------------
// Reoit le contenu des registres de la carte MD23/MD25. Liste ci-dessous (dcrite dans MD23.h)
// Speed1, Speed2_Turn, Enc1a, Enc1b, Enc1c, Enc1d, Enc2a, Enc2b, Enc2c, Enc2d, Ubatt, IMot1, IMot2, SoftRev,AccRate,Mode,Cde
unsigned char Table_Registers_MD23[]={MD23_ADD_BASE_R,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
// ===========================================================================================================================
// 2) Gestion du LCD BATRON
// ---------------------------------------------------------------------------------------------------------------------------
char Buffer_for_LCD_Line0[17], Buffer_for_LCD_Line1[17];        // Reoivent les deux lignes de texte  envoyer  l'afficheur
															    // (16 caractres max. + Nul)
// ===========================================================================================================================
// 3) Gestion de la carte SD21
// ---------------------------------------------------------------------------------------------------------------------------
unsigned char Vitesse = 5;				    // Vitesse de dplacement du servo          
unsigned int tempo = 1000;                  // Attente entre deux transmission : doit tre > 20ms 
unsigned int i = 0;						    // Variable de boucle 
// ===========================================================================================================================
// 4) Gestion de la boussole HMC6352
// ---------------------------------------------------------------------------------------------------------------------------
// Variables utilises pour la gestion de l'affichage sur le LCD:  limitent son rafrachissement!
unsigned char Direction_MSB_1 = 0, Direction_LSB_1 = 0;
// ---------------------------------------------------------------------------------------------------------------------------
unsigned char Direction_MSB = 0,  Direction_LSB = 0;    // Valeur de l'angle renvoy par la boussole HMC6352
unsigned int AngleEnDegreFoisDix=0;                     // Calcul de l'angle dlivr par la boussole en 1/1000 soit
                                                        // AngleEnDegreFoisDix = 256*MSB + LSB
char AfficheAngle[] = {"0000"};                         // Reprsentation textuelle de AngleEnDegreFoisDix
// --------------------------------------------------------------------------------------------------------------------------
// ===========================================================================================================================


// ===========================================================================================================================
// ===================================================== Initialisations =====================================================
// ===========================================================================================================================
// Initialisation de l'interface TWI en mode Standby
// ---------------------------------------------------------------------------------------------------------------------------
TWI_Master_Initialise(); 									    
// ---------------------------------------------------------------------------------------------------------------------------
// Global enable interrupts     // ATTENTION !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
#asm("sei")					    // L'autorisation des interruptions doit se faire aprs l'initialisation de l'interface TWI
// ===========================================================================================================================
TWI_Start_Transc_With_Data(Frame_I2C_LED_All_Off,sizeof(Frame_I2C_LED_All_Off)); // Extinction des LEDs de la carte SISI
// ===========================================================================================================================
// Initialisation de l'afficheur I2C BATRON
// ---------------------------------------------------------------------------------------------------------------------------
Init_LCD_I2C_BATRON();
// ===========================================================================================================================
// Initialisation des registres de  la carte MD23
// ---------------------------------------------------------------------------------------------------------------------------
TWI_Start_Transc_With_Data(Reset_Encodeurs_MD23,sizeof(Reset_Encodeurs_MD23));
//TWI_Start_Transc_With_Data(Time_OUT_Desable,sizeof(Time_OUT_Desable)); // Dsactivation de la scurit moteur
TWI_Start_Transc_With_Data(Mode0_MD23,sizeof(Mode0_MD23));
TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2));
// ===========================================================================================================================
// Affichage d'un message d'accueil
// ---------------------------------------------------------------------------------------------------------------------------
sprintf(Buffer_for_LCD_Line0,Texte_Accueil_Ligne1);
sprintf(Buffer_for_LCD_Line1,Texte_Accueil_Ligne2);
Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
delay_ms(4000);
// ===========================================================================================================================

// ===========================================================================================================================
// ================================================ Boucle principale ========================================================
// ===========================================================================================================================
while (1)
      {
// ==========================================================================================================================
// ---------------------------------------------  Champ Lecture des donnes -------------------------------------------------
// Lecture de l'tat des boutons-poussoir (BP)
// --------------------------------------------------------------------------------------------------------------------------
// Lecture des BP pour affichage sur les LED
	TWI_Start_Transc_With_Data(Frame_I2C_BP_Read,1);          // Envoi de l'octet d'adresse des BP: SLA+R=0x43
// Rcupration de l'octet reprsentatif de l'tat des BP
	TWI_Get_Data_From_Transceiver(Frame_I2C_BP_Read,sizeof(Frame_I2C_BP_Read));
// Lecture des BP avec gestion anti-rebond pour parcourir les menus
    BP_I2C = Lire_BP_I2C();
// --------------------------------------------------------------------------------------------------------------------------
// Lecture des registres de la carte MD23
   if (Menu != Reglage_Valeurs_Mode0_MD23)
   {                                                         // Envoi de l'octet d'adresse MD23 + bit RW=criture + n du Reg
	TWI_Start_Transc_With_Data(Read_Reg_MD23_W,sizeof(Read_Reg_MD23_W));
   	TWI_Start_Transc_With_Data(Table_Registers_MD23,18);  		// Envoi octet d'adresse MD23 +  bit RW=lecture
	TWI_Get_Data_From_Transceiver(Table_Registers_MD23,sizeof(Table_Registers_MD23));
   }
// ==========================================================================================================================
// --------------------------------------------------- Champ Traitement------------------------------------------------------
    switch(Menu)
    {// Dbut du switch1
// -------------------------------------------------Dbut Menus droulants---------------------------------------------------
// Dbut des menus Niveau 1 -------------------------------------------------------------------------------------------------
// ==========================================================================================================================
// Menu [1] : Choix du test des Led et des BP (Recopie de l'tat des BP sur les LED)
// ==========================================================================================================================
       case Menu1_LED_BP : if (Menu_1 != Menu)
       		{
                sprintf(Buffer_for_LCD_Line0,TexteMenuLedBp);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEntr);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
       		}
       		switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Copie_BP_Sur_Led;  break;     // touche Enter
					case FlecheH_I2C: Menu = Menu4_HMC6352;  break;     // touche flche haut
					case FlecheB_I2C: Menu = Menu2_MD23;  break;        // touche flche bas
					default: Menu = Menu1_LED_BP;
				}
			Menu_1 = Menu1_LED_BP;
       break;
// ==========================================================================================================================
// Menu [2] : Choix des test de la carte MD23/25 
// ==========================================================================================================================
       case Menu2_MD23 : if (Menu_1 != Menu)
       		{
                sprintf(Buffer_for_LCD_Line0,TexteMenuMD23);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEntr);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
       		}
       		switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Menu20_Mode_MD23;  break;     // touche Enter
					case FlecheH_I2C: Menu = Menu1_LED_BP;  break;      // touche flche haut
					case FlecheB_I2C: Menu = Menu3_SD21;  break;        // touche flche bas
					default: Menu = Menu2_MD23 ;
				}
			Menu_1 = Menu2_MD23 ;
       break;
// ==========================================================================================================================
// Menu [3] : Choix du test de la carte SD21
// ==========================================================================================================================
       case Menu3_SD21 : if (Menu_1 != Menu)
       		{
                sprintf(Buffer_for_LCD_Line0,TexteMenuSD21);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEntr);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
       		}
       		switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Test_SD21;  break;           // touche Enter
					case FlecheH_I2C: Menu = Menu2_MD23;  break;       // touche flche haut
					case FlecheB_I2C: Menu = Menu4_HMC6352;  break;    // touche flche bas
					default: Menu = Menu3_SD21 ;
				}
			Menu_1 = Menu3_SD21 ;            
       break; 
// ==========================================================================================================================
// Menu [4] : Choix du test de la boussole HMC6352
// ==========================================================================================================================
       case Menu4_HMC6352 : if (Menu_1 != Menu)
       		{
                sprintf(Buffer_for_LCD_Line0,TexteMenuHMC6352);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEntr);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
       		}
       		switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Test_HMC6352;  break;         // touche Enter
					case FlecheH_I2C: Menu = Menu3_SD21;  break;        // touche flche haut
					case FlecheB_I2C: Menu = Menu1_LED_BP;  break;      // touche flche bas
					default: Menu = Menu4_HMC6352 ;
				}
			Menu_1 = Menu4_HMC6352 ;            
       break;
// --------------------------------------------------------------------------------------------------------------------------
// Fin des menus niveau 1
// ==========================================================================================================================
// Sous menus(Niveau 2) -----------------------------------------------------------------------------------------------------
// ==========================================================================================================================
// Sous menu [20]: Choix du mode de commande des moteurs de la carte MD23/25
// ==========================================================================================================================
		case Menu20_Mode_MD23 : if (Menu_1 != Menu)
			{
				sprintf(Buffer_for_LCD_Line0,Ligne0MenuMode);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchapOk);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				LCD_I2C_BATRON_gotoxy(0,15);
				Built_Strng_Fram_I2C_LCDBATRON(Frame_I2C_Lcd_Line, Buffer_for_LCD_Line0, strlen(Buffer_for_LCD_Line0));
				TWI_Start_Transc_With_Data(Frame_I2C_Lcd_Line,strlen(Frame_I2C_Lcd_Line));
				Display_Arrow_0n_I2C_LCDBATRON();
			}

			switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Choix_Mode_MD23;  break;         // touche Enter
					case FlecheH_I2C: Menu = Menu22_Deplacements;  break;  // touche flche haut
					case FlecheB_I2C: Menu = Menu21_Infos_MD23;  break;    // touche flche bas
					case ECHAP_I2C : Menu = Menu2_MD23; break;             // touche ECHAP
					default: Menu = Menu20_Mode_MD23 ;
				}
			Menu_1 = Menu20_Mode_MD23;
		break;
// ==========================================================================================================================
// Sous menu [21] : Choix de l'affichage du contenu des registres de la carte MD23/25
// ==========================================================================================================================
		case Menu21_Infos_MD23: if (Menu_1 != Menu)
			{
				sprintf(Buffer_for_LCD_Line0,Ligne0MenuInfos);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchapOk);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
			}

			switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Info_All;  break;
					case FlecheH_I2C: Menu = Menu20_Mode_MD23;  break;
					case FlecheB_I2C: Menu = Menu22_Deplacements;  break;
					case ECHAP_I2C : Menu = Menu2_MD23; break; 
					default: Menu = Menu21_Infos_MD23 ;
				}

			Menu_1 = Menu21_Infos_MD23 ;
		break;
// ==========================================================================================================================
// Sous menu [22] : Choix d'un test de dplacement
// ==========================================================================================================================
		case Menu22_Deplacements: if (Menu_1 != Menu)
			{
				sprintf(Buffer_for_LCD_Line0,Ligne0MenuDeplacements);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchapOk);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
			}

			switch (BP_I2C)
				{
					case ENTR_I2C: Menu = Selection_Deplacements;  break;
					case FlecheH_I2C: Menu = Menu21_Infos_MD23;  break;
					case FlecheB_I2C: Menu = Menu20_Mode_MD23;  break;
					case ECHAP_I2C : Menu = Menu2_MD23; break;              
					default: Menu = Menu22_Deplacements ;
				}
			Menu_1 = Menu22_Deplacements ;
		break;
// --------------------------------------------------------------------------------------------------------------------------          
// Fin des sous menus
// ==========================================================================================================================
// Tests --------------------------------------------------------------------------------------------------------------------
// ==========================================================================================================================
// Test associ  [1]: Affichage du nom du BP et de la LED  laquelle il est associ sur le LCD
// ==========================================================================================================================
			case Copie_BP_Sur_Led : 
              if ((Menu_1 != Menu) || ((BP_I2C != BP_I2C_1) && (BP_I2C != 0xFF)))
				{
				 sprintf(Buffer_for_LCD_Line0,TexteTest1BpsurLed);

				 switch (BP_I2C)
				  {
				 	case ENTR_I2C : sprintf(Buffer_for_LCD_Line1,BpEntrSurD6); break;
				 	case FlecheH_I2C : sprintf(Buffer_for_LCD_Line1,BpFlecheHSurD1); break;
				 	case FlecheB_I2C : sprintf(Buffer_for_LCD_Line1,BpFlecheHSurD2); break;
				 	case ECHAP_I2C : sprintf(Buffer_for_LCD_Line1,BpEchSurD7); break;
				 	case OK_I2C : sprintf(Buffer_for_LCD_Line1,BpOkSurD5); break;
				 	case MOINS_I2C: sprintf(Buffer_for_LCD_Line1,BpMoinsSurD3); break;
				 	case PLUS_I2C: sprintf(Buffer_for_LCD_Line1,BpPlusSurD0); break;
				 	case SET_I2C: sprintf(Buffer_for_LCD_Line1,BpSetSurD4); break;
				 	case 0 : sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchap );
				  }

					Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
					BP_I2C_1 = BP_I2C;
			   	 }

        		if (BP_I2C == ECHAP_I2C)
        		  {
        			Frame_I2C_LED_Switching[1] = ~Frame_I2C_BP_Read[1];
        			TWI_Start_Transc_With_Data(Frame_I2C_LED_Switching,2);  	// Les LED affichent l'tat des BP
        			delay_ms(1000);
        			Menu = Menu1_LED_BP;
        		  }
        		else  Menu = Copie_BP_Sur_Led;

			    Menu_1 = Copie_BP_Sur_Led;
			break;
// ==========================================================================================================================
// Rglage(s) associ(s) au sous-menu "[20] Choix_Modes" : Rglage du mode de commande des moteurs 
// (Le premier rglage est "[200] Mode 0"
// ==========================================================================================================================
	        case Choix_Mode_MD23: if ((Menu_1 != Menu) || (Choix_Mode_1 != Choix_Mode))
	          {
	        	 sprintf(Buffer_for_LCD_Line0,"Choix: Mode%-u    ",Choix_Mode);
	        	 sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchapOk);
	        	 Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
	        	 Display_Arrow_0n_I2C_LCDBATRON();
	        	 Choix_Mode_1 = Choix_Mode;
	          }

	          switch (BP_I2C)
				{
					case ECHAP_I2C : Menu=Menu20_Mode_MD23; break;  // touche ECHAP
					case FlecheH_I2C: Choix_Mode = Choix_Mode - 1; if (Choix_Mode < 0) Choix_Mode=3;  break; 
					case FlecheB_I2C:  Choix_Mode = Choix_Mode + 1; if (Choix_Mode > 3) Choix_Mode=0 ;  break; 
					case ENTR_I2C: if (Choix_Mode == 0) 
                                        {SpeedMot1 = 128; SpeedMot2 = 128; Menu = Reglage_Valeurs_Mode0_MD23;}	
                    break;

					default: Menu=Choix_Mode_MD23;
				}

			  Menu_1 = Choix_Mode_MD23;
	        break;
// ==========================================================================================================================
            case Reglage_Valeurs_Mode0_MD23:   if ((Menu_1 != Menu) || (SpeedMot1_1 != SpeedMot1)||(SpeedMot2_1 != SpeedMot2)
                                                                    ||(Finesse_1 != Finesse))
              {
                sprintf(Buffer_for_LCD_Line0,"[200]Mode%-u (+%-u) ",Choix_Mode,Finesse);
                sprintf(Buffer_for_LCD_Line1,"SM1=%-u SM2=%-u",SpeedMot1,SpeedMot2);
                Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
                Display_Arrow_0n_I2C_LCDBATRON();
                SpeedMot1_1 = SpeedMot1; SpeedMot2_1 = SpeedMot2; Finesse_1 = Finesse;
              }


              switch (BP_I2C)
				{
				 	case ECHAP_I2C :  Menu=Choix_Mode_MD23; break;

				 	case PLUS_I2C:
				 		switch (Choix_Moteur)
				 		  {
				 		   	case MoteurG: if (Finesse==1)
				 		   					    if (SpeedMot1 < 255) SpeedMot1++; else SpeedMot1=255;
				 		   			      else // Finesse == 0
				 		   					    if (SpeedMot1 < 250) SpeedMot1+=5; else SpeedMot1=255;
				 		   	break;
				 		   	case MoteurD: if (Finesse==1)
				 		   					    if (SpeedMot2 < 255) SpeedMot2++; else SpeedMot2=255;
				 		   	              else // Finesse == 0
				 		   					    if (SpeedMot2 < 250) SpeedMot2+=5; else SpeedMot2=255;
				 		   	break;
				 		  }
				 	break;

				 	case MOINS_I2C:
				 	    switch (Choix_Moteur)
				 		  {
				 		   	case MoteurG: if (Finesse==1)
				 		   					    if (SpeedMot1 > 0) SpeedMot1--; else SpeedMot1=0;
				 		   			      else  // Finesse == 0
				 		   					    if (SpeedMot1 > 5) SpeedMot1-=5; else SpeedMot1=0;
				 		   	break;
				 		   	case MoteurD: if (Finesse==1)
				 		   					    if (SpeedMot2 > 0) SpeedMot2--; else SpeedMot2=0;
				 		   			      else  // Finesse == 0
				 		   					    if (SpeedMot2 > 5) SpeedMot2-=5; else SpeedMot2=0;
				 			break;
				 		  }
				 	break;

				 	case FlecheH_I2C: if (Finesse==1) Finesse = 5; else Finesse = 1; break;

				 	case FlecheB_I2C: if (Finesse==1) Finesse = 5; else Finesse = 1; break;

				 	case OK_I2C: if (Choix_Moteur == 1) Choix_Moteur=0; else Choix_Moteur=1; break;

				 	case SET_I2C:   Speed1[2] = SpeedMot1; Speed2[2] = SpeedMot2;
				 				    TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
									TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2));
				    break;

					default: Menu = Reglage_Valeurs_Mode0_MD23;
				}
			  Menu_1 = Reglage_Valeurs_Mode0_MD23;
			break;
// ==========================================================================================================================
// Affichages associ au sous-menu "[21] Affichage ..."  (Contenu des registres de la carte MD23 ou MD25)
// ==========================================================================================================================
// Affichage de tous les registres de la carte MD23/MD25 (associ  "[210] Affichage registres")
		case Info_All: Vitesse_MD23 = Table_Registers_MD23[6]; 
			if ((Menu_1 != Menu)||(Num_Reg_MD23_1 != Num_Reg_MD23)||(Vitesse_MD23_1 != Vitesse_MD23))
			  {
				sprintf(Buffer_for_LCD_Line0,"[210] Reg%-u= %-u",Num_Reg_MD23,Table_Registers_MD23[Num_Reg_MD23+1]) ;
				strcpyf(Buffer_for_LCD_Line1,Ligne1PlusMoins);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
				Num_Reg_MD23_1 = Num_Reg_MD23; Vitesse_MD23_1 = Vitesse_MD23;
			  }

			switch (BP_I2C)
				{
					case ECHAP_I2C : Menu=Menu21_Infos_MD23; break;
					case PLUS_I2C: if (Num_Reg_MD23<16) Num_Reg_MD23++; else Num_Reg_MD23=0;  break;
					case MOINS_I2C: if (Num_Reg_MD23>0) Num_Reg_MD23--; else Num_Reg_MD23=16;  break;
					case FlecheH_I2C: Menu = Info_TensionBatt;  break;
					case FlecheB_I2C: Menu = Info_Version;  break;
					default: Menu=Info_All;
				}

			Menu_1 = Info_All;
		break;
// ==========================================================================================================================
// Affichage de la version du logiciel implant dans la carte MD23 ou MD25 (associ  "[211] Affichage version soft")
// ==========================================================================================================================
		case Info_Version:	Version = Table_Registers_MD23[14];
			if ((Menu_1 != Menu)||(Version_1 != Version))
   			  {
				sprintf(Buffer_for_LCD_Line0,"[211] Vers.=%-u  ",Version);
				strcpyf(Buffer_for_LCD_Line1,Ligne1ChoixEchap);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
				Display_Arrow_0n_I2C_LCDBATRON();
				Version_1 = Version;
			  }

			switch (BP_I2C)
				{
					case ECHAP_I2C : Menu=Menu21_Infos_MD23; break;
					case FlecheH_I2C: Menu = Info_All;  break;
					case FlecheB_I2C: Menu = Info_TensionBatt;  break;
					default: Menu=Info_Version;
				}

			Menu_1 = Info_Version;
		break;
// ==========================================================================================================================
// Affichage de la tension d'alimentation de la carte MD23 ou MD25  (associ  "[212] Affichage tension batterie")
// ==========================================================================================================================
		case Info_TensionBatt:  	Ubatt = Table_Registers_MD23[11];
			if ((Menu_1 != Menu)||(Ubatt_1 != Ubatt))
   			  {
					sprintf(Buffer_for_LCD_Line0,"[212]Ubat=%-u.%-uV ",Ubatt/10, Ubatt-10*(Ubatt/10));
					strcpyf(Buffer_for_LCD_Line1,Ligne1ChoixEchap);
					Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
                    Display_Arrow_0n_I2C_LCDBATRON();
                    Ubatt_1 = Ubatt;
			  }

			switch (BP_I2C)
				{
					case ECHAP_I2C : Menu=Menu21_Infos_MD23; break;
					case FlecheH_I2C: Menu = Info_Version;  break;
					case FlecheB_I2C: Menu = Info_All;  break;
					default: Menu=Info_TensionBatt;
				}

			Menu_1 = Info_TensionBatt;
		break;
// ==========================================================================================================================
// Slection d'un type de dplacement associ au sous-menu "[22] Choix dplacement"(La premire slection est "Ligne droite")
// ==========================================================================================================================
		case Selection_Deplacements:  	
			if ((Menu_1 != Menu)||(SelecDeplacement_1 != SelecDeplacement))
   			  {
                    switch (SelecDeplacement)
                    {
					    case 0: sprintf(Buffer_for_LCD_Line0,"[220]Lig. droit "); break;
                        case 1: sprintf(Buffer_for_LCD_Line0,"[221]Virage G90 "); break;
                        case 2: sprintf(Buffer_for_LCD_Line0,"[222]Vira. G180 "); break;
                        case 3: sprintf(Buffer_for_LCD_Line0,"[223]Virage D90 "); break;
                        case 4: sprintf(Buffer_for_LCD_Line0,"[224]Vira. D180 "); break;
                        case 5: sprintf(Buffer_for_LCD_Line0,"[225]Bala. Zone "); break;
                        case 6: sprintf(Buffer_for_LCD_Line0,"[226]Orien. Sud "); break;
                        default: sprintf(Buffer_for_LCD_Line0,"[220]Lig. droit ");
                    }
					strcpyf(Buffer_for_LCD_Line1,Ligne1ChoixEchapSET);
					Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
                    Display_Arrow_0n_I2C_LCDBATRON();
                    SelecDeplacement_1 = SelecDeplacement;
			  }

			switch (BP_I2C)
				{
					case ECHAP_I2C : Menu = Menu22_Deplacements; break;
					case FlecheH_I2C: if (SelecDeplacement > 0) SelecDeplacement--; else SelecDeplacement=6;  break;
					case FlecheB_I2C: if (SelecDeplacement < 6) SelecDeplacement++; else SelecDeplacement=0;  break;
                    case SET_I2C:   
                        switch (SelecDeplacement) // Rglage des vitesses (sera remplac par l'appel d'une fonction)
                          { // Seul le Mode 0 est gr !!!!!!!!!!!!!!!!! 
                            // Paramtrage du test slectionn
                            // Exemple calcul avec fichier xls
                            // Consigne vitesse moyenne  = 20cm/s, Distance de la ligne droite = 1m, 
                            // Rayon du virage = 23 cm
                            // Remarque: Slectionner le codeur correspondant  la plus grande valeur de Speed  
                            // Sur le robot:
                            // Speed1 = Registre vitesse moteur G - Speed 2 = Registre vitesse moteur D
                            // Numero codeur = 0 pour l'encodeur 1 (gauche) et Numero codeur = 1 pour l'encodeur 2 (droit)
                            case 0: Speed1[2] = 151; Speed2[2] = 151; Valeur_Codeur_Stop = 1156; NumeroCodeur = 0;  break;  // Dplacement en ligne droite   
                            case 1: Speed1[2] = 128; Speed2[2] = 173; Valeur_Codeur_Stop = 828; NumeroCodeur = 1; break;    // Virage  90  G
                            case 2: Speed1[2] = 128; Speed2[2] = 173; Valeur_Codeur_Stop = 1656; NumeroCodeur = 1; break;   // Virage  180  G 
                            case 3: Speed1[2] = 173; Speed2[2] = 128; Valeur_Codeur_Stop = 828; NumeroCodeur = 0; break;    // Virage  90  D
                            case 4: Speed1[2] = 173; Speed2[2] = 128; Valeur_Codeur_Stop = 1656; NumeroCodeur = 0; break;   // Virage  180  D
                            case 5: // Balayage 
                                    // Initialisation 
                                    Direction_MSB = 0; Direction_LSB = 0; AngleEnDegreFoisDix = 0;
                                    i = 0; // Initialisation du premier mouvement stock dans TableBalayage
                                    delay_ms(3000); // Attente avant le dpart et reset des encodeurs au cas ou on dplace
                                    // le robot avant de partir
                                    TWI_Start_Transc_With_Data(Reset_Encodeurs_MD23,sizeof(Reset_Encodeurs_MD23));
                                    Speed1[2] = Table_Balayage[i][0]; Speed2[2] = Table_Balayage[i][1];
                                    Valeur_Codeur_Stop = Table_Balayage[i][2]; NumeroCodeur = Table_Balayage[i][3]; 
                                    TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
									TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2));
                                    Menu = Balayage;
                            break;
                            case 6:    delay_ms(3000); Menu = Orientation_Sud; break;
                            default :  Speed1[2] = 128; Speed2[2] = 128;           // Arrt des moteurs en mode 0   
                          }
                                    
                            if (SelecDeplacement < 5)
                                { 
                                    delay_ms(3000); // Attente avant le dpart et reset des encodeurs au cas ou on dplace
                                    // le robot avant de partir
                                    TWI_Start_Transc_With_Data(Reset_Encodeurs_MD23,sizeof(Reset_Encodeurs_MD23));
                                    TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
								    TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2));
                                    Menu = Test_Deplacements_Elementaires;
                                }  
                                        
                    break;
					default: Menu = Selection_Deplacements;
				}

			Menu_1 = Selection_Deplacements;
		break;
// ==========================================================================================================================
// Gestion d'un dplacement lmentaire(associ  "[220] Dplacement ligne droite" ou "[221] Virage 90" ou [222] Virage 180)
// ==========================================================================================================================
		case Test_Deplacements_Elementaires:
            // Calcul du dplacement en nombre de priodes du codeur
            TableCodeur[0] = (long int)65536*Table_Registers_MD23[4] + (long int)256*Table_Registers_MD23[5] + Table_Registers_MD23[6];
            TableCodeur[1] = (long int)65536*Table_Registers_MD23[8] + (long int)256*Table_Registers_MD23[9] + Table_Registers_MD23[10];
            
                                                                        // Test si la distance prvue a t parcourue
            if (TableCodeur[NumeroCodeur] >= Valeur_Codeur_Stop)                                                                        
             {                                                                  
                Speed1[2] = 128; Speed2[2] = 128;  // Arrt                         
			    TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
			    TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2)); 
             } 
                                                                        // Affichage pour visualisation sur le banc Moteurs
            else  if ((Menu_1 != Menu) || (TableCodeur_1[NumeroCodeur] != TableCodeur[NumeroCodeur]))	                                                                  
			            
   			             { 
                            ltoa(TableCodeur[0], AfficheDeplacement1); // Conversion du dplacement en une chane de
                            ltoa(TableCodeur[1], AfficheDeplacement2); // caractres avant affichage sur le LCD
                            strcpy(Buffer_for_LCD_Line0,"Enc1=");                       
                            strcat(Buffer_for_LCD_Line0, AfficheDeplacement1);
					        strcpyf(Buffer_for_LCD_Line1,"Enc2=");
                            strcat(Buffer_for_LCD_Line1, AfficheDeplacement2);
                            strcat(Buffer_for_LCD_Line1," [Echa]");
					        Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
                            TableCodeur_1[NumeroCodeur] = TableCodeur[NumeroCodeur];
			             }
                 
			switch (BP_I2C)
				  {
					case ECHAP_I2C : Menu = Selection_Deplacements;
                                     Speed1[2] = 128; Speed2[2] = 128;   // Arrt des moteurs
				                     TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
				                     TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2));
                                     delay_ms(500); // Arrt complet des moteurs !
                                     // Remise  zro des encodeurs
                                     TWI_Start_Transc_With_Data(Reset_Encodeurs_MD23,sizeof(Reset_Encodeurs_MD23)); 
                    break;
					default: Menu = Test_Deplacements_Elementaires;
				  }

			Menu_1 = Test_Deplacements_Elementaires;
		break;
// ==========================================================================================================================
// Gestion du balayage d'une zone de 16m et orientation associ  "[223] Balayage Zone"
// ==========================================================================================================================
        case Balayage: 
// --------------------------------------------------------------------------------------------------------------------------                    
        // Squence de dplacement 
// --------------------------------------------------------------------------------------------------------------------------
         for (i = 0 ; i <  Nombre_Deplacements_du_Balayage - 1;i++)
          {
           do
           {
// --------------------------------------------------------------------------------------------------------------------------
            // Lecture des codeurs  
// --------------------------------------------------------------------------------------------------------------------------
            // Envoi de l'octet d'adresse MD23 + bit RW=criture + n du Reg
	            TWI_Start_Transc_With_Data(Read_Reg_MD23_W,sizeof(Read_Reg_MD23_W));
   	            TWI_Start_Transc_With_Data(Table_Registers_MD23,18);  		// Envoi octet d'adresse MD23 +  bit RW=lecture
	            TWI_Get_Data_From_Transceiver(Table_Registers_MD23,sizeof(Table_Registers_MD23));
// --------------------------------------------------------------------------------------------------------------------------
            // Calcul du dplacement en nombre de priodes du codeur 
// --------------------------------------------------------------------------------------------------------------------------
            TableCodeur[0] = (long int)65536*Table_Registers_MD23[4] + (long int)256*Table_Registers_MD23[5] + Table_Registers_MD23[6];
            TableCodeur[1] = (long int)65536*Table_Registers_MD23[8] + (long int)256*Table_Registers_MD23[9] + Table_Registers_MD23[10]; 
// --------------------------------------------------------------------------------------------------------------------------
             ltoa(TableCodeur[0], AfficheDeplacement1);                      // Conversion du dplacement en une chane de
             ltoa(TableCodeur[1], AfficheDeplacement2);                      // caractres avant affichage sur le LCD
             strcpy(Buffer_for_LCD_Line0,"Enc1=");                       
             strcat(Buffer_for_LCD_Line0, AfficheDeplacement1);
			 strcpyf(Buffer_for_LCD_Line1,"Enc2=");
             strcat(Buffer_for_LCD_Line1, AfficheDeplacement2);
             strcat(Buffer_for_LCD_Line1," [Echa]");
			 Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
             TableCodeur_1[NumeroCodeur] = TableCodeur[NumeroCodeur];
           } 
            while (TableCodeur[NumeroCodeur] <= Valeur_Codeur_Stop);        // Fin Dplacement(i)
            
// --------------------------------------------------------------------------------------------------------------------------                                                                      
            //Speed1[2] = 128; Speed2[2] = 128;                               // Initialisation Dplacement i + 1
			//TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
			//TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2)); 
            //delay_ms(500);  
            // Initialisation du  nouveau dplacement -> transfert des vitesses vers la carte moteur -> Reset codeur et
            // Attente avant de returner dans la boucle                                                      
            Speed1[2] = Table_Balayage[i+1][0]; Speed2[2] = Table_Balayage[i+1][1]; // Initialisation Dplacement i + 1
            Valeur_Codeur_Stop = Table_Balayage[i+1][2]; NumeroCodeur = Table_Balayage[i+1][3];                            
			TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));    // Transfert
			TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2));
            TWI_Start_Transc_With_Data(Reset_Encodeurs_MD23,sizeof(Reset_Encodeurs_MD23)); // Reset encodeurs
            delay_ms(500); // Evite perturbation sur les roues lors des changement de vitesse
            } 
// --------------------------------------------------------------------------------------------------------------------------
// --------------------------------------------------------------------------------------------------------------------------            
         // Orientation au sud
// --------------------------------------------------------------------------------------------------------------------------         
         // Attente avant positionnement au sud  
             delay_ms(3000);
             Menu = Orientation_Sud;
// --------------------------------------------------------------------------------------------------------------------------           
		 Menu_1 = Balayage;
		break;
// ==========================================================================================================================
// Orientation du panneau au sud
// ==========================================================================================================================
		case Orientation_Sud:  
// --------------------------------------------------------------------------------------------------------------------------    
            // Lecture de la boussole HMC6352 pour connaitre la position initiale du robot 
// --------------------------------------------------------------------------------------------------------------------------
            // Ecriture de la commande 'A'
            TWI_Start_Transc_With_Data(Frame_I2C_Write_Command_HMC6352, sizeof(Frame_I2C_Write_Command_HMC6352)); 
            delay_ms(10); // time delay constructeur
            // Lecture des octets MSB et LSB
            TWI_Start_Transc_With_Data(Frame_I2C_Read_Data_HMC6352,sizeof(Frame_I2C_Read_Data_HMC6352));
            TWI_Get_Data_From_Transceiver(Frame_I2C_Read_Data_HMC6352,sizeof(Frame_I2C_Read_Data_HMC6352)); 
            Direction_MSB = Frame_I2C_Read_Data_HMC6352[1]; Direction_LSB = Frame_I2C_Read_Data_HMC6352[2];
// --------------------------------------------------------------------------------------------------------------------------
            //  Traitement (choix du sens de dplacement
// --------------------------------------------------------------------------------------------------------------------------
            // Calcul de l'angle 
            AngleEnDegreFoisDix = MSLSBYTEtoUINT(Direction_MSB,Direction_LSB);
              
            // Configuration des registres des moteurs
            if (AngleEnDegreFoisDix > 1800)
                {
                 Speed1[2] = 139; Speed2[2] = 117;  // Rotation anti-horaire 
                }
            else
                { 
                 Speed1[2] = 117; Speed2[2] = 139; // Rotation horaire
                }
            TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
		    TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2));
// --------------------------------------------------------------------------------------------------------------------------
            // Affichage du rsultat 
// --------------------------------------------------------------------------------------------------------------------------
            do
              {
                // Ecriture sur le lcd  
                itoa(AngleEnDegreFoisDix,AfficheAngle); 
                if ((Direction_MSB_1 != Direction_MSB)||(Direction_LSB_1 != Direction_LSB)) 
                    {	// Affichage des deux octets
	                    sprintf(Buffer_for_LCD_Line0,"MSB=%-x LSB=%-x",Direction_MSB, Direction_LSB);
	                    Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,AfficheAngle); // Affichage de l'angle  
	                    Display_Arrow_0n_I2C_LCDBATRON();
	                    Direction_MSB_1 = Direction_MSB; Direction_LSB_1 = Direction_LSB; 
                    } 
// --------------------------------------------------------------------------------------------------------------------------    
                // Lecture de la boussole HMC6352 pour connaitre la position initiale du robot 
// --------------------------------------------------------------------------------------------------------------------------
                // Ecriture de la commande 'A'
                TWI_Start_Transc_With_Data(Frame_I2C_Write_Command_HMC6352, sizeof(Frame_I2C_Write_Command_HMC6352));
                delay_ms(10); // time delay constructeur
                // Lecture des octets MSB et LSB
                TWI_Start_Transc_With_Data(Frame_I2C_Read_Data_HMC6352,sizeof(Frame_I2C_Read_Data_HMC6352));
                TWI_Get_Data_From_Transceiver(Frame_I2C_Read_Data_HMC6352,sizeof(Frame_I2C_Read_Data_HMC6352)); 
                Direction_MSB = Frame_I2C_Read_Data_HMC6352[1]; Direction_LSB = Frame_I2C_Read_Data_HMC6352[2];
                // Calcul de l'angle 
                AngleEnDegreFoisDix = MSLSBYTEtoUINT(Direction_MSB,Direction_LSB);    
                // Communication avec la carte MD23 (sinon arrt au bout de 2s lorsque la scurit est active
                TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
		        TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2)); 
              }
             while (( AngleEnDegreFoisDix < 3550) && ( AngleEnDegreFoisDix > 50));
// --------------------------------------------------------------------------------------------------------------------------
             // Arrt des moteurs
             Speed1[2] = 128; Speed2[2] = 128;                          
			 TWI_Start_Transc_With_Data(Speed1,sizeof(Speed1));
			 TWI_Start_Transc_With_Data(Speed2,sizeof(Speed2));
             delay_ms(1000); // Arrt complet des moteurs !
             Menu = Test_HMC6352;  // Pour conserver l'affichage de l'orientation du robot
// --------------------------------------------------------------------------------------------------------------------------
			Menu_1 = Orientation_Sud ;
       break;
// ==========================================================================================================================
// Test de la boussole
// ==========================================================================================================================
       case Test_HMC6352 : 
// --------------------------------------------------------------------------------------------------------------------------    
            // Lecture de la boussole HMC6352 
// --------------------------------------------------------------------------------------------------------------------------
            // Ecriture de la commande 'A'
            TWI_Start_Transc_With_Data(Frame_I2C_Write_Command_HMC6352, sizeof(Frame_I2C_Write_Command_HMC6352));
            delay_ms(10); // time delay constructeur
            // Lecture des octets MSB et LSB
            TWI_Start_Transc_With_Data(Frame_I2C_Read_Data_HMC6352,sizeof(Frame_I2C_Read_Data_HMC6352));
            TWI_Get_Data_From_Transceiver(Frame_I2C_Read_Data_HMC6352,sizeof(Frame_I2C_Read_Data_HMC6352)); 
            Direction_MSB = Frame_I2C_Read_Data_HMC6352[1]; Direction_LSB = Frame_I2C_Read_Data_HMC6352[2];
// --------------------------------------------------------------------------------------------------------------------------
            // Traitement
// --------------------------------------------------------------------------------------------------------------------------
            // Inversion de l'tat logique des BP pour affichage sur les leds
            Frame_I2C_LED_Switching[1] = ~Frame_I2C_BP_Read[1]; 
            // Calcul de l'angle et conversion pour affichage 
            AngleEnDegreFoisDix = MSLSBYTEtoUINT(Direction_MSB,Direction_LSB);
            itoa(AngleEnDegreFoisDix,AfficheAngle);
// --------------------------------------------------------------------------------------------------------------------------
            // Affichage du rsultat 
// --------------------------------------------------------------------------------------------------------------------------
            // Ecriture sur le lcd  
            if ((Direction_MSB_1 != Direction_MSB)||(Direction_LSB_1 != Direction_LSB)) 
                {	// Affichage des deux octets
	                sprintf(Buffer_for_LCD_Line0,"MSB=%-x LSB=%-x",Direction_MSB, Direction_LSB);
	                Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,AfficheAngle); // Affichage de l'angle  
	                Display_Arrow_0n_I2C_LCDBATRON();
	                Direction_MSB_1 = Direction_MSB; Direction_LSB_1 = Direction_LSB; 
                }  

            delay_ms(100); // pour affichage stable sur le LCD
// --------------------------------------------------------------------------------------------------------------------------
       		switch (BP_I2C)
				{
					case ECHAP_I2C: Menu = Menu4_HMC6352;  break;       // touche Enter
					case FlecheH_I2C: Menu = Menu3_SD21;  break;        // touche flche haut
					case FlecheB_I2C: Menu = Menu1_LED_BP;  break;      // touche flche bas
					default: Menu = Test_HMC6352; ;
				}

			Menu_1 = Test_HMC6352; ;           
       break;
// --------------------------------------------------------------------------------------------------------------------------
// ==========================================================================================================================
// Commande de quatre servos relis  la carte SD21  associ au menu "[3] Test SD21"
// ==========================================================================================================================
        case Test_SD21 : if (Menu_1 != Menu)
       		{
                sprintf(Buffer_for_LCD_Line0,TexteTestSD21);
				sprintf(Buffer_for_LCD_Line1,Ligne1ChoixEchap5s);
				Displ_2Lines_On_I2C_LCDBATRON(Buffer_for_LCD_Line0,Buffer_for_LCD_Line1);
       		}  
            
            // Position initial du servo1
            RegSpeedValue = Vitesse; RegPosLByte= 0xE8; RegPosHByte=0x03;  					// 1000s = 03E8h
            TWI_Start_Transc_With_Data(Frame_I2C_Servo,sizeof(Frame_I2C_Servo));			// Vitesse, LSByte, MSByte                                                                                                                                           
            // ===============================================================================================================
            // ================================================ Boucle  ======================================================
            // =============================================================================================================== 
            // Pour info
            // 1000 = 3E8h	valeur min
            // 1250 = 4E2h
            // 1500 = 5DCh
            // 1750 = 6D6h 
            // 1950 = 7E9h et + ?
            // ---------------------------------------------------------------------------------------------------------------
            // Les valeurs "n Servo + Position" transmises  la carte SD21 par le bus I2C doivent tre places dans 
            // les tableaux situs dans le fichier "TableRob.h"
            // Le nom des servos est dclar dans le fichier sd21.h  
            // ---------------------------------------------------------------------------------------------------------------
            
            while (BP_I2C != ECHAP_I2C) // Test des servos tant que l'on n'appuie pas sur "ECHAP"
            { 
                // Configuration des paramtres "Servox" [0<x<5] et "position" envoys aux servos 
                for(i=0; i<sizeof(Tab_Servo); i++)
                {
                // Configuration de la trame "Frame_I2C_Servo" avec les valeurs contenues dans les tableaux situs dans 
                // le fichier TableRob.h
                NumServo = Tab_Servo[i]; RegPosLByte = LSBYTE(Tab_Position[i]);  RegPosHByte = MSBYTE(Tab_Position[i]); 
     	
	            // Envoi de la trame  "Frame_I2C_Servo"  la carte SD21
	            TWI_Start_Transc_With_Data(Frame_I2C_Servo,sizeof(Frame_I2C_Servo));	      
	
	            // Visualisation de la position des valeurs places dans les tableaux sur les Led de la carte Sisi
	            Frame_I2C_LED_Switching[1] = ~(i+1);
	            TWI_Start_Transc_With_Data(Frame_I2C_LED_Switching,sizeof(Frame_I2C_LED_Switching));
	            delay_ms(tempo);   
                } // fin du for
                 
                BP_I2C = Lire_BP_I2C();    
            } 
            // Servo maintenus dans leur dernire position
            RegSpeedValue = 0; TWI_Start_Transc_With_Data(Frame_I2C_Servo,sizeof(Frame_I2C_Servo));
                  
			Menu = Menu3_SD21 ; Menu_1 = Test_SD21 ;
            
       break; 
// ========================================================================================================================== 
    }// Fin du switch1

    Frame_I2C_LED_Switching[1] = ~Frame_I2C_BP_Read[1]; // Inversion de l'tat logique des BP pour affichage sur les leds
// --------------------------------------------------------------------------------------------------------------------------
// ----------------------------------------  Champ Ecriture des rsultats  --------------------------------------------------
// Ecriture sur les LED
    TWI_Start_Transc_With_Data(Frame_I2C_LED_Switching,sizeof(Frame_I2C_LED_Switching));  // Les LED affichent l'tat des BP
// --------------------------------------------------------------------------------------------------------------------------
//delay_ms(100);   // A utiliser pour visualiser les trames I2C
  }; // Fin while(1)

} //Fin main
// ==========================================================================================================================
// ==========================================================================================================================


